#!perl
# Tests command line invocations of opsview_api

package cmdline;
use Class::Accessor::Fast;
use base qw(Class::Accessor::Fast);
__PACKAGE__->mk_accessors(qw(output rc));

use Test::More;

use XML::LibXML;
my $rngschema = XML::LibXML::RelaxNG->new( location => "/usr/local/nagios/share/xml/opsview.rng" );
my $parser = XML::LibXML->new;

# I think we have to have this hard coded - needs more thinking about
my $cmd = "/usr/local/nagios/bin/opsview_api";

my $Bin = $FindBin::Bin;

sub test {
    my $class = shift;
    my $self  = $class->new;
    my $exec  = "$cmd @_";
    local $/ = undef;
    open F, "$exec |" or die "Cannot invoke '$exec'";
    $self->output( $_ = <F> );
    close F;
    $self->rc( $? >> 8 );
    eval { $_ = $rngschema->validate( $parser->parse_string( $self->output ) ) };
    is( $@, "", "Result validated okay" ) || diag("Full output:".$self->output);
    $self;
}

package main;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/lib", "$FindBin::Bin/../lib", "$FindBin::Bin/../etc", "$FindBin::Bin/../../opsview-core/t/lib", "$FindBin::Bin/../../opsview-core/lib", "$FindBin::Bin/../../opsview-core/etc";

use Opsview::Test;
use Opsview;
use Opsview::Host;
use Opsview::XML;

use Test::More tests => 65;
use Test::LongString;
use Test::Differences;

my $t;
my $host;
my $expected;

$t = cmdline->test("-c");
is( $t->rc, 1, "Bad option" );
like( $t->output, "/Incorrect options.*OPTIONS-ERROR/s", "With correct XML returned data" );

$t = cmdline->test("");
is( $t->rc, 1, "Missing file" );
like( $t->output, "/Must specify an XML file.*OPTIONS-ERROR/s", "With correct XML returned data" );

$t = cmdline->test("-f missingfile PUT /api");
is( $t->rc, 1, "Missing file" );
like( $t->output, "/File 'missingfile' does not exist.*OPTIONS-ERROR/s", "With correct XML data" );

my $file;    # Try to choose a file that is not readable
foreach $_ (qw(/etc/sudoers /etc/shadow)) {
    $file = $_;
    last if -e $file;
}
$t = cmdline->test("-f $file PUT /api");
is( $t->rc, 1, "Unreadable file" );
like( $t->output, "/Cannot open '$file'.*OPTIONS-ERROR/s", "With correct XML data" );

open F, "> $Bin/xml.test" or die "Cannot create xml.test file";
print F <<'EOF';
<opsview><authentication><username>admin</username><password>initial</password></authentication>
<host action="create">
<name>testviaxml</name>
<check_command><name>ping</name></check_command>
</host>
</opsview>
EOF
close F;

$t = cmdline->test("-f $Bin/xml.test");
is( $t->rc, 1, "Missing fields in creation" );
like( $t->output, "/Field ip is mandatory.*FAILED/s", "With correct XML data" );

open F, "> $Bin/xml.test" or die "Cannot create xml.test file";
print F <<'EOF';
<opsview><authentication><username>admin</username><password>initial</password></authentication>
<host action="create">
<name>testviaxml</name>
<check_command><name>ping</name></check_command>
<ip>10.11.12.13</ip> <hostgroup><name>Monitoring Servers</name></hostgroup>
<icon><name>LOGO - Nokia</name></icon>
</host>
</opsview>
EOF
close F;

$t = cmdline->test("-f $Bin/xml.test");
is( $t->rc, 0, "Created" );
like( $t->output, "/OK/s", "With correct XML data" );

open F, "> $Bin/xml.test" or die "Cannot create xml.test file";
print F <<'EOF';
<opsview><authentication><username>admin</username><password>initial</password></authentication>
<host action="create">
<name>testviaxml</name>
<check_command><name>ping</name></check_command>
<ip>10.11.12.13</ip> <hostgroup><id>1</id></hostgroup>
<icon><name>LOGO - Nokia</name></icon>
</host>
</opsview>
EOF
close F;

$t = cmdline->test("-f $Bin/xml.test");
is( $t->rc, 1, "Failed when hostgroup id = 1 - must be a leaf" );
like( $t->output, "/hostgroup fails.*must_be_leaf.*constraint.*FAILED/s", "With correct XML data" );

open F, "> $Bin/xml.test" or die "Cannot create xml.test file";
print F <<'EOF';
<opsview><authentication><username>admin</username><password>initial</password></authentication>
<host action="create">
<name>testviaxml2</name>
<check_command><name>ping</name></check_command>
<ip>10.11.12.13</ip> <hostgroup><id>156989</id></hostgroup>
<icon><name>LOGO - Nokia</name></icon>
</host>
</opsview>
EOF
close F;

$t = cmdline->test("-f $Bin/xml.test");
is( $t->rc, 1, "Failed when hostgroup id is not valid" );
like( $t->output, "/Cannot find object hostgroup with id=156989.*FAILED/s", "With correct XML data" );

open F, "> $Bin/xml.test" or die "Cannot create xml.test file";
print F <<'EOF';
<opsview><authentication><username>admin</username><password>initial</password></authentication>
<host action="delete" by_id="2"/>
</opsview>
EOF
close F;

# DELETE tests
$t = cmdline->test("-f $Bin/xml.test");
is( $t->rc, 0, "Deleted" );
like( $t->output, "/OK/", "With correct XML data" );

# Test create host
$t = cmdline->test("-f $Bin/api/full_create_host.xml");
is( $t->rc, 0, "Created" );
like( $t->output, "/OK/", "With correct XML data" );

# Test create same host
$t = cmdline->test("-f $Bin/api/full_create_host.xml");
is( $t->rc, 1, "Failed same hostname" );
like( $t->output, "/Can't insert new Opsview::CRUD::Host: DBD::mysql::st execute failed: Duplicate entry 'fullcreatehost'/", "With correct XML data" );

TODO: {
    local $TODO = "Some problems at the moment with exceptions and snmpinterfaces";

    # Test create host
    $t = cmdline->test("-f $Bin/api/full_create_host_todo.xml");
    is( $t->rc, 0, "Created" );
    like( $t->output, "/OK/", "With correct XML data" );

    # Test create same host
    $t = cmdline->test("-f $Bin/api/full_create_host_todo.xml");
    is( $t->rc, 1, "Failed same hostname" );
    like( $t->output, "/Can't insert new Opsview::CRUD::Host: DBD::mysql::st execute failed: Duplicate entry 'fullhostcreate'/", "With correct XML data" );

}

# Test bad characters
$t = cmdline->test("-f $Bin/api/bad_characters_in_hostname.xml");
is( $t->rc, 1, "Failure due to bad characters in hostname" );
like( $t->output, "/validate_column_values error: name Opsview::Host name fails 'regexp' constraint/", "With correct failure" );

$host = Opsview::Host->search( name => "toclone" )->first;
isa_ok( $host, "Opsview::Host", "Found host to clone" );
is( $host->servicecheckexceptions->count,              1, "One exception" );
is( $host->servicechecktimedoverrideexceptions->count, 1, "One timed exception" );

# Test clone ignores exceptions and timedexceptions
$t = cmdline->test("-f $Bin/api/clone_host.xml");
is( $t->rc, 0, "Clone okay" );
like( $t->output, "/OK/", "XML back" );

$host = Opsview::Host->search( name => "clone_of_toclone" )->first;
isa_ok( $host, "Opsview::Host", "Found newly cloned host" );
is( $host->servicecheckexceptions->count,              0,              "No exceptions" );
is( $host->servicechecktimedoverrideexceptions->count, 0,              "No timed exceptions either" );
is( $host->icon,                                       "SYMBOL - Proxy", "But logo is of the same" );

# Test invalid servicecheck id
$t = cmdline->test("-f $Bin/api/invalid_servicecheckid.xml");
is( $t->rc, 1, "Failure due to invalid servicecheck id" );
like( $t->output, "/Cannot find object servicechecks with id=28282828282/", "With correct failure" );
is( Opsview->db_Main->selectrow_array("SELECT COUNT(*) FROM hosts WHERE name='badspud'"), 0, "Confirmed that host is not created" );

open F, "> $Bin/reload.test" or die "Cannot create reload.test file";
print F <<'EOF';
<opsview><authentication><username>admin</username><password>initial</password></authentication>
<system action="reload"/>
</opsview>
EOF
close F;

# RELOAD test - expect failure first time
$t = cmdline->test("-f $Bin/reload.test");
is( $t->rc, 1, "Reloaded" );


# There's a funny \r character (I think) at end of "not known"
# We remove the line that has opslave as sometimes it says "lost connection" and other times it says "cannot connect"
my $cleaned = $t->output;
$cleaned =~ s%detail>ClusterA:critical:ssh: .*?opslave.*?</deta%%s;
# So deliberately looks like <il> below
# Also, we convert opslaveclusterAip, opslaveclusterBip and C to opslavecluster?ip as the order could be different - as long as they
# exist, this is okay
$cleaned =~ s/opslavecluster[ABC]ip/opslavecluster?ip/g;
unified_diff;
eq_or_diff( $cleaned,
qq{<opsview>
  <result>
    <detail>Master Monitoring Server:warning:Missing required variable PAGER for contact somehosts for notification method AQL - ignoring this notification</detail>
    <detail>Master Monitoring Server:warning:Unable to lookup IP address for 'cisco4' (not_a_real_host) - ignoring SNMP trap configuration for this host</detail>
    <detail>Master Monitoring Server:warning:IP 192.168.10.22 has more than 1 host associated with it: cisco2, cisco3</detail>
    <detail>ClusterA:warning:Missing required variable PAGER for contact somehosts for notification method AQL - ignoring this notification</detail>
    <detail>ClusterA:warning:Unable to lookup IP address for 'cisco4' (not_a_real_host) - ignoring SNMP trap configuration for this host</detail>
    <detail>ClusterA:warning:IP 192.168.10.22 has more than 1 host associated with it: cisco2, cisco3</detail>
    <il>
    <detail>Cluster:warning:Missing required variable PAGER for contact somehosts for notification method AQL - ignoring this notification</detail>
    <detail>Cluster:warning:Unable to lookup IP address for 'cisco4' (not_a_real_host) - ignoring SNMP trap configuration for this host</detail>
    <detail>Cluster:warning:IP 192.168.10.22 has more than 1 host associated with it: cisco2, cisco3</detail>
    <detail>Cluster:critical:ssh: Could not resolve hostname opslavecluster?ip: Name or service not known
lost connection
</detail>
    <detail>Cluster:critical:ssh: Could not resolve hostname opslavecluster?ip: Name or service not known
lost connection
</detail>
    <detail>Cluster:critical:ssh: Could not resolve hostname opslavecluster?ip: Name or service not known
lost connection
</detail>
    <detail>PassiveSlave:warning:Missing required variable PAGER for contact somehosts for notification method AQL - ignoring this notification</detail>
    <detail>PassiveSlave:warning:Unable to lookup IP address for 'cisco4' (not_a_real_host) - ignoring SNMP trap configuration for this host</detail>
    <detail>PassiveSlave:warning:IP 192.168.10.22 has more than 1 host associated with it: cisco2, cisco3</detail>
    <status>FAILED</status>
  </result>
</opsview>
}, "With correct XML data - this is linked to hudson's output so may be different on a dev server" );


# TODO: Need an API to remove a monitoringserver
# RELOAD test - should work this time
#$t = cmdline->test("-f $Bin/reload.test");
#is( $t->rc, 0, "Reloaded" );
#like( $t->output, '/OK/', "Correct output" );

# TODO: Possible an asynchronous reload and then test that we get a clash?

unlink( "$Bin/xml.test", "$Bin/reload.test" );

# This causes a test error since the return message from the api isnt xml
#open my $fh, "> $Bin/get.test" or die "Cannot create get.test.file: $!";
#print $fh <<'EOF';
#<opsview><authentication><username>blah</username><password>blah</password></authentication></opsview>
#EOF
#close $fh;
#
#$expected = qq{You are not authorised to use the status API};
#$t = cmdline->test("-f $Bin/get.test GET api/status/hostgroup");
#like_string($t->output, qr/$expected/ms, "status hostgroup GET incorrect auth ok");

open my $fh, "> $Bin/get.test" or die "Cannot create get.test.file: $!";
print $fh <<'EOF';
<opsview><authentication><username>admin</username><password>initial</password></authentication></opsview>
EOF
close $fh;

# shouldn't need full test or data here as that should be done elsewhere already
$expected = qq{<opsview>
  <result>
    <detail>XML data required</detail>
    <status>OPTIONS-ERROR</status>
  </result>
</opsview>
};
$t = cmdline->test("-f $Bin/get.test GET api/status/rubbish");
is_string( $t->output, $expected, "status rubbish GET ok" );

$expected = qq{<opsview>
  <data name="hostgroup">
  .*
  </data>
</opsview>
};
$t = cmdline->test("-f $Bin/get.test GET api/status/hostgroup");
like_string( $t->output, qr/$expected/ms, "status hostgroup GET ok" );

$expected = qq{<opsview>
  <data name="service">
  .*
  </data>
</opsview>
};
$t = cmdline->test("-f $Bin/get.test GET api/status/service");
like_string( $t->output, qr/$expected/ms, "status service GET ok" );

unlink( "$Bin/xml.test", "$Bin/get.test" );
